import datetime
from urllib3.util import Retry

import requests
from requests.adapters import HTTPAdapter


TEST_ENDPOINT = 'https://api-test.dhl.com/track'
PRODUCTION_ENDPOINT = 'https://api-eu.dhl.com/track'


class DHLApi():
  def __init__(self, api_key, environment='production'):
    self.endpoint = TEST_ENDPOINT
    self.environment = environment
    if environment == 'production':
      self.endpoint = PRODUCTION_ENDPOINT

    self.api_key = api_key
    self.session = requests.Session()
    retries = Retry(
      total=7,
      backoff_factor=0.1,
      status_forcelist=[502, 503, 504],
    )
    self.session.mount(self.endpoint, HTTPAdapter(max_retries=retries))

  def get_tracking_details(self, tracking_number, service=None, origin_country_code=None, requester_country_code=None):
    url = "{}/shipments".format(self.endpoint)

    query = {
      "trackingNumber": tracking_number,
    }
    if service:
      query['service'] = service

    if origin_country_code:
      query['originCountryCode'] = origin_country_code

    if requester_country_code:
      query['requesterCountryCode'] = requester_country_code

    headers = {
      "Accept": 'application/json',
      "DHL-API-Key": self.api_key,
    }

    response = self.session.get(url, headers=headers, params=query)
    return response.json()
