import json
import datetime
from urllib3.util import Retry

import requests
from requests.adapters import HTTPAdapter


TEST_ENDPOINT = 'https://apis-sandbox.fedex.com'
PRODUCTION_ENDPOINT = 'https://apis.fedex.com'


class FedexApi():
  def __init__(self, client_id, client_secret, environment='production', grant_type='client_credentials'):
    self.endpoint = TEST_ENDPOINT
    self.environment = environment
    if environment == 'production':
      self.endpoint = PRODUCTION_ENDPOINT

    self.client_id = client_id
    self.client_secret = client_secret
    self.grant_type = grant_type
    self.token = None
    self.session = requests.Session()
    retries = Retry(
      total=7,
      backoff_factor=0.1,
      status_forcelist=[429, 502, 503, 504],
    )
    self.session.mount(self.endpoint, HTTPAdapter(max_retries=retries))
    self.authorize()

  def get_tracking_details(self, tracking_number):
    url = "{}/track/v1/trackingnumbers".format(self.endpoint)

    payload = {
      'includeDetailedScans': True,
      'trackingInfo': [{'trackingNumberInfo': {'trackingNumber': tracking_number}}]
    }

    headers = {
      "X-locale": "en_US",
      "Content-Type": "application/json",
      "Authorization": "Bearer {}".format(self.token['access_token'])
    }

    response = self.session.post(url, headers=headers, data=json.dumps(payload))
    return response.json()

  def authorize(self):
    url = "{}/oauth/token".format(self.endpoint)

    payload = {
      "client_id": self.client_id,
      "client_secret": self.client_secret,
      "grant_type": self.grant_type
    }

    headers = {
      "Content-Type": "application/x-www-form-urlencoded",
    }

    response = self.session.post(
      url, data=payload, headers=headers)
    self.token = response.json()

    if not self.token or 'access_token' not in self.token or not self.token['access_token']:
      raise RuntimeError("Authorization failed")
